{~~ This unit contains the TIB_CursorGrid component. }
unit IB_CursorGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids,

  IB_Header,
  IB_Components;

type
{~~...
This component acts as a buffer of fetched data.}
  TIB_CursorGrid = class(TStringGrid)
  private
  { Property Storage }
    FIB_DataLink: TIB_DataLink;
    FFrozen: boolean;
  { Property Access Methods }
    procedure SetDataSource( AValue: TIB_DataSource); virtual;
    function GetDataSource: TIB_DataSource; virtual;
    procedure SetFrozen( AValue: boolean ); virtual;
  { IB_CursorLink methods }
    procedure IB_StateChanged( Sender: TIB_DataLink;
                               DataSource: TIB_DataSource );
    procedure IB_CursorRowDataChange( Sender: TIB_DataLink;
                                      DataSource: TIB_DataSource;
                                      IB_Field: TIB_Column );
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
  protected
  { Inherited Methods }
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  { Utility Methods }
    procedure UpdateGrid;
    procedure CloseSQLCursorGrid;
    procedure ClearSQLCursorGrid;
    procedure FillSQLCursorGrid;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
{~~$<*#>...
This property freezes the grid so that it does not respond to changes in ...
the underlying dataset.

This can be useful if the user fetches all the rows and then wants ...
to be able to double-click on any of the fetched rows and have that row ...
re-fetched for editing without clearing the previously fetched batch of rows.

This could be accomplished by setting Frozen to true just after the last ...
row is fetched. Frozen could be set to false just prior to another ...
request to go to the last record in a selected dataset. This way the grid ...
would always represent a complete sub-set of data. Then, using this sub-set ...
the user can select and work with one record at a time by double-clicking ...
on them in the grid.}
    property Frozen: boolean read FFrozen write SetFrozen;
  published
{~~$<*>...
Reference to the DataSource for the grid.}
    property DataSource: TIB_DataSource read GetDataSource write SetDataSource;
  end;

implementation

constructor TIB_CursorGrid.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FIB_DataLink := TIB_DataLink.Create( Self );
  with FIB_DataLink do begin
    OnStateChanged := IB_StateChanged;
    OnDataChange := IB_CursorRowDataChange;
  end;
  Options := Options - [goEditing];
  Options := Options - [goColMoving];
  FixedCols := 0;
end;

destructor TIB_CursorGrid.Destroy;
begin
  DataSource := nil;
  with FIB_DataLink do begin
    OnStateChanged := nil;
    OnDataChange  := nil;
  end;
  inherited Destroy;
end;

procedure TIB_CursorGrid.Loaded;
begin
  inherited Loaded;
  UpdateGrid;
end;

{------------------------------------------------------------------------------}

function TIB_CursorGrid.GetDataSource: TIB_DataSource;
begin
  Result := FIB_DataLink.DataSource;
end;

procedure TIB_CursorGrid.SetDataSource( AValue: TIB_DataSource);
begin
  if FIB_DataLink.DataSource <> AValue then begin
    FIB_DataLink.DataSource := AValue;
    UpdateGrid;
  end;
end;

procedure TIB_CursorGrid.SetFrozen( AValue: boolean );
begin
  if Frozen <> AValue then begin
    FFrozen := AValue;
    if Frozen = false then begin
      UpdateGrid;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_CursorGrid.IB_StateChanged( Sender: TIB_DataLink;
                                          DataSource: TIB_DataSource);
begin
  UpdateGrid;
end;

procedure TIB_CursorGrid.IB_CursorRowDataChange( Sender: TIB_DataLink;
                                                 DataSource: TIB_DataSource;
                                                 IB_Field: TIB_Column);
begin
  if Frozen or ( DataSource = nil ) then Exit;
  with DataSource do begin
    if DataSet <> nil then with DataSet do begin
      if BOF then begin
        ClearSQLCursorGrid;
      end else if EOF and Unidirectional then begin
      end else begin
        FillSQLCursorGrid;
      end;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_CursorGrid.UpdateGrid;
var
  ii: integer;
begin
  if Frozen or ( DataSource = nil ) then Exit;
  if not ( csLoading    in ComponentState ) and
     not ( csDestroying in ComponentState ) then begin
    case DataSource.State of
      dsInactive: CloseSQLCursorGrid;
      dsPrepared: ClearSQLCursorGrid;
    end;
    if FIB_DataLink.Prepared then begin
      with DataSource.DataSet do begin
        ColCount := FieldCount;
        for ii := 0 to FieldCount - 1 do begin
          Cells[ ii, 0 ] := Fields[ ii ].DisplayName;
        end;
      end;
    end else begin
      CloseSQLCursorGrid;
    end;
  end;
end;

{------------------------------------------------------------------------------}
{   Grid handling procedures                                                   }
{------------------------------------------------------------------------------}

procedure TIB_CursorGrid.CloseSQLCursorGrid;
var
  ii: integer;
begin
  RowCount := 2;
  FixedRows := RowCount - 1;
  RowCount := FixedRows + 1;
  for ii := 0 to FixedRows do begin
    Rows[ ii ].Clear;
  end;
  ColCount := 1;
end;

procedure TIB_CursorGrid.ClearSQLCursorGrid;
var
  ii: integer;
  OldCursor: TCursor;
begin
  OldCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourglass;
    with DataSource.DataSet do begin
      for ii := FixedRows to RowCount - 1 do begin
        Rows[ ii ].Clear;
      end;
      Self.RowCount := FixedRows + 1;
    end;
    if ( FIB_DataLink.Prepared ) then begin
      with DataSource.DataSet do begin
        ColCount := FieldCount;
        for ii := 0 to FieldCount - 1 do with Fields[ ii ] do begin
          ColWidths[ ii ] := DisplayWidth;
        end;
      end;
    end;
  finally
    Screen.Cursor := OldCursor;
  end;
end;

procedure TIB_CursorGrid.FillSQLCursorGrid;
var
  Index: integer;
begin
  if ( FIB_DataLink.Active ) then begin
    with DataSource.DataSet do begin
      if Self.RowCount < RowNum + FixedRows then begin
        Self.RowCount := RowNum + FixedRows;
      end;
      for Index := 0 to FieldCount - 1 do begin
        if BOF then begin
        end else if EOF then begin
          Cells[ Index, RowNum + FixedRows - 1] := '';
        end else if Fields[ Index ].IsNull then begin
          Cells[ Index, RowNum + FixedRows - 1] := '<NULL>';
        end else if Fields[ Index ].IsBlob then begin
          Cells[ Index, RowNum + FixedRows - 1] := '(BLOB)';
        end else begin
          Cells[ Index, RowNum + FixedRows - 1] := Fields[ Index ].DisplayText;
        end;
      end;
    end;
  end;
end;

procedure TIB_CursorGrid.CMEnter( var Message: TCMEnter );
begin
  FIB_DataLink.SetFocus;
  inherited;
end;

procedure TIB_CursorGrid.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if FIB_DataLink.Prepared then with DataSource.DataSet do begin
    case Key of
      VK_DOWN: begin
        if not EOF and (( RowNum = Row ) or BOF ) then Next;
      end;
    end;
  end;
  inherited KeyDown( Key, Shift );
  if FIB_DataLink.Prepared then with DataSource.DataSet do begin
    if ssCtrl in Shift then case Key of
      VK_HOME: begin
        if not Active or BOF then First;
        Row := FixedRows;
        Key := 0;
      end;
      VK_END: begin
        if not EOF then Last;
        Row := RowCount - FixedRows;
        Key := 0;
      end;
    end;
  end;
end;

end.
