
{                                                                              }
{   TIB_FieldDataLink                                                          }
{                                                                              }

unit
  IB_FieldDataLink;

interface

uses
  Classes, Graphics,

  IB_Header,
  IB_Components,
  IB_Parse;

type
  TIB_FieldDataLink = class( TIB_DataLink )
  private
    FFieldName: string;
    FField: TIB_Column;
    procedure SetFieldName( AValue: string ); virtual;
    procedure SetFieldText( AValue: string ); virtual;
    function GetFieldText: string; virtual;
    function GetInvalidText: string; virtual;
    function GetFieldAlignment: TAlignment; virtual;
    function GetFieldEditmask: string; virtual;
    function GetFieldMaxLength: integer; virtual;
    function GetReadOnly: boolean; override;
    function GetColor: TColor; override;
    function GetEditingColor: TColor; override;
    function GetInsertingColor: TColor; override;
    procedure SetSearchBuffer( AValue: string ); override;
  protected
    procedure SysDataChange( AField: TIB_Column ); override;
    procedure SysUpdateData( AField: TIB_Column ); override;
    procedure SysAfterUpdateDescriptors; override;
    procedure DoPrepareSQL; override;
    procedure DoAfterAssignment; override;
    function Modify: boolean; override;
    procedure SysAssignField; virtual;
    procedure DoAssignField; virtual;
  public
    OnAssignField: TIB_ColumnDataLinkEvent;
    procedure UpdateData; override;
    procedure FocusControl( AField: TIB_ColumnRef ); override;
    function FieldIsValidChar( AKey: char ): boolean; virtual;
    property FieldName: string read FFieldName write SetFieldName;
    property Field: TIB_Column read FField;
    property FieldText: string read GetFieldText write SetFieldText;
    property FieldAlignment: TAlignment read GetFieldAlignment;
    property FieldEditMask: string read GetFieldEditmask;
    property FieldMaxLength: integer read GetFieldMaxLength;
  end;

implementation

function TIB_FieldDataLink.FieldIsValidChar( AKey: char ): boolean;
begin
  Result := ( Field <> nil ) and Prepared and 
            (( State = dsSearch ) or Field.IsValidChar( AKey ));
end;

procedure TIB_FieldDataLink.SetFieldText( AValue: string );
begin
  case State of
    dsEdit,
    dsInsert: Field.AsString := AValue;
    dsSearch: SearchBuffer   := AValue;
  end;
end;

function TIB_FieldDataLink.GetFieldText: string;
begin
  if not Prepared or ( Field = nil ) then begin
    Result := GetInvalidText;
  end else case State of
    dsPrepared: Result := '';
    dsBrowse: if ( DataSet <> nil ) and
                 ( DataSet.Fields.RowState <> rsNone ) then begin
      if ( Control <> nil ) and Control.Focused then begin
        Result := Field.AsString
      end else begin
        Result := Field.DisplayText;
      end;
    end else begin
      Result := '';
    end;
    dsEdit,
    dsInsert,
    dsDelete: if ( Control <> nil ) and Control.Focused then begin
      Result := Field.AsString
    end else begin
      Result := Field.DisplayText;
    end;
    dsSearch: Result := SearchBuffer;
    else begin
      Result := GetInvalidText;
    end;
  end;
end;

function TIB_FieldDataLink.GetInvalidText: string;
begin
  if ( csDesigning in ComponentState ) and
     ( State = dsInactive ) and
     ( Control <> nil ) then begin
    Result := Control.Name;
  end else begin
    Result := '';
  end;
end;

function TIB_FieldDataLink.GetFieldAlignment: TAlignment;
begin
  if not Prepared or ( Field = nil ) or ( State = dsSearch ) then begin
    Result := taLeftJustify;
  end else begin
    Result := Field.Alignment;
  end;
end;

function TIB_FieldDataLink.GetFieldEditMask: string;
begin
  if ( Field <> nil ) and ( State in [ dsBrowse,
                                       dsEdit,
                                       dsInsert,
                                       dsDelete ] ) then begin
    Result := Field.EditMask;
  end else begin
    Result := '';
  end;
end;

function TIB_FieldDataLink.GetFieldMaxLength: integer;
begin
  if not Prepared or ( Field = nil ) or ( State = dsSearch ) then begin
    Result := 0;
  end else begin
    if ( Field.SQLType = SQL_Varying  ) or
       ( Field.SQLType = SQL_Varying_ ) or
       ( Field.SQLType = SQL_Text     ) or
       ( Field.SQLType = SQL_Text_    ) then begin
      Result := Field.SQLLen;
    end else begin
      Result := 0;
    end;
  end;
end;

function TIB_FieldDataLink.GetReadOnly: boolean;
begin
  Result := ( Field = nil ) or
            ( Field.ReadOnly and ( State <> dsSearch )) or
            inherited GetReadOnly;
  if not Result then case State of
    dsEdit:   Result := Field.IsAttributeSet[ 'NOEDIT'   ];
    dsInsert: Result := Field.IsAttributeSet[ 'NOINSERT' ];
    dsSearch: Result := Field.IsAttributeSet[ 'NOSEARCH' ];
  end;
end;

function TIB_FieldDataLink.GetColor: TColor;
begin
  if not Prepared or ( Field = nil ) then begin
    Result := GetInvalidColor;
  end else begin
    Result := inherited GetColor;
  end;
end;

function TIB_FieldDataLink.GetEditingColor: TColor;
begin
  if Field.IsNull then begin
    Result := inherited GetEditingColor; // or clSilver;
  end else begin
    Result := inherited GetEditingColor;
  end;
end;

function TIB_FieldDataLink.GetInsertingColor: TColor;
begin
  if Field.IsNull then begin
    Result := inherited GetInsertingColor; // or clSilver;
  end else begin
    Result := inherited GetInsertingColor;
  end;
end;

procedure TIB_FieldDataLink.SetSearchBuffer( AValue: string );
begin
  if SearchBuffer <> AValue then begin
    inherited SetSearchBuffer( AValue );
    if DataSet <> nil then begin
      DataSet.InvalidateSQL;
    end;
    if State = dsSearch then begin
      SysDataChange( Field );
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_FieldDataLink.DoAfterAssignment;
begin
  SysAssignField;
  inherited DoAfterAssignment;
end;

procedure TIB_FieldDataLink.SetFieldName( AValue: string );
begin
  if FieldName <> AValue then begin
    FFieldName := AValue;
    SysAssignField;
  end;
end;

procedure TIB_FieldDataLink.SysDataChange( AField: TIB_Column );
begin
  if ( AField = nil ) or ( AField = Field ) then begin
    inherited SysDataChange( Field );
  end;
end;

procedure TIB_FieldDataLink.SysUpdateData( AField: TIB_Column );
begin
  if ( ControlIsModified or ( State = dsSearch )) and
     (( AField = nil ) or ( AField = Field )) then begin
    inherited SysUpdateData( Field );
  end;
end;

procedure TIB_FieldDataLink.SysAfterUpdateDescriptors;
begin
  if not DataSet.RepreparingSQL then begin
    SysAssignField;
  end;
  inherited SysAfterUpdateDescriptors;
end;

function TIB_FieldDataLink.Modify: boolean;
begin
  if ReadOnly then begin
    Result := false;
  end else begin
    Result := inherited Modify;
  end;
end;

procedure TIB_FieldDataLink.SysAssignField;
var
  OldField: TIB_Column;
begin
  OldField := Field;
  if DataSet <> nil then begin
    DataSet.Fields.GetByName( FieldName, FField );
  end else begin
    FField := nil;
  end;
  if OldField <> FField then begin
    DoAssignField;
  end;
end;

procedure TIB_FieldDataLink.DoAssignField;
begin
  if not ( csLoading in ComponentState ) and
     not ( csDestroying in ComponentState ) then begin
    if Assigned( OnAssignField ) then begin
      OnAssignField( Self, DataSource, Field );
    end else begin
      SysStateChanged;
      SysDataChange( Field );
    end;
  end;
end;

procedure TIB_FieldDataLink.DoPrepareSQL;
begin
  if Assigned( OnPrepareSQL ) then begin
    inherited DoPrepareSQL;
  end else if DataSet <> nil then with DataSet do begin
    if SearchBuffer <> '' then begin
      if Field <> nil then begin
        if Field.IsIndexField then begin
          ExtractWhereClause( Field,
                              SearchBuffer,
                              SQLWhereHigh,
                              SQLWhereParams );
        end else begin
          ExtractWhereClause( Field,
                              SearchBuffer,
                              SQLWhereMed,
                              SQLWhereParams );
        end;
      end;
    end;
  end;
end;

procedure TIB_FieldDataLink.UpdateData;
begin
  if Field <> nil then begin
    SysUpdateData( Field );
  end;
end;

procedure TIB_FieldDataLink.FocusControl( AField: TIB_ColumnRef ); 
begin
  if (AField^ <> nil) and
     (AField^ = Field) and
     (Control <> nil) and
     Control.CanFocus then begin
    AField^ := nil;
    Control.SetFocus;
  end;
end;

end.
