unit hash;

(*
  class THash 1.0

  Luu Tran <luutran@geocities.com>

  Last revised: 8-Jul-97

  This is a free component.  Use it as you like, at your own risk.
  Please send me any bug fix and improvement you make.

  ------------------------

  THash: Implement open hashing with separate chaining.

  THash works like Perl's associative array, i.e.,
  an array indexed by a Key of type string with an
  associated Value of type Pointer, which means Value
  is anything that will fit into a Pointer type -- you just
  need to do the appropriate typecasting.

  There is a default hash size (number of buckets) of 17,
  so you can start adding items right away if you wish.  By
  default, I double the hash size and rebuild it whenever
  number of items > 2 x number of buckets.  The rebuild is
  expensive and best avoided by setting an appropriate
  hash size at the outset.  As usual, there's a speed vs.
  space tradeoff.  Note: you can turn off the automatic
  resize feature by setting AutoGrow to false.

  You can add an item or change its value by simply
  assigning to the hash.  (Just like an array)
  You can delete an item by assigning NIL as its value
  or use the explicit .Delete method.
  You can also explicitly add with the Add method.
  The Add method does not check whether the item
  already exists so it is slightly faster; it's useful
  when you are populating the hash at the start.
  There is also an explicit Delete method.

  Examples:

  zip:=THash.Create;
  zip.Size:=100;
  zip[ 'Pasadena']:=Pointer( 91107);
  City:='Boston';
  zip[ City]:=Pointer( 02935);

  TStudent = class
    Name: string;
    Age: integer;
    Major: string;
  end;

  students:=THash.Create;
  students.size:=967;
  std:=TStudent.Create;
  with std do
  begin
    Name:='jack';
    Age:=30;
    Major:='cs';
  end;
  students[std.Name]:=std;

  Freeing items:

  As with TList, you are responsible for freeing the value
  objects in the hash.  Hence, you need to free std in the
  example above.

  Walking through the hash:

  You can walk through the hash with the .First and .Next method.
  Adding or removing items while walking through the hash
  yields unpredictable results.

  Example:

  var HI:THashItem;
  HI:=students.First;
  while HI.Value<>nil do
  begin
    with TStudent( HI.Value) do
      writeln( 'Name = '+Name+'; Age = '+IntToStr( Age));
    HI:=students.Next;
  end;

  Custom hash function:

  THash has a simple hash function which may or may not suit
  you.  In the latter case, you can specify your own
  hash function (e.g., MyHash.HashFunc:=MyHashFunc).  The
  hash function must a) take a string Key and return a
  longint hash value; and b) return the same hash value for
  a given Key (of course).  (I always take the absolute
  value so it doesn't matter if the hash function returns
  a negative number.)  Needless to say, the efficiency
  of the hash hinges on selecting a mapping function such
  that the input set is distributed evenly among the range
  0..MAXINT-1.  Note: assigning a hash function after you
  have added some items to the hash will force a rebuild.

  Statistic properties:

  There are several statistic properties to help you evaluate
  your hash function and choice of hash sizes:

  Count: number of hash items.
  Used: number of used buckets.  You want Count/Used to be as
    close to 1 as possible.  What's more, you want this ratio
    to be more or less constant regardless of number of hash items.
  Chains: number of chains, i.e., buckets with more than 1 item.
  LongestChain: size of the longest chain.
  AvgChainSize: number of items in chains / number of chains.
  iCount[ B]: number of items in bucket B.

  Implementation notes:

  The hash is a TStringList (FHash).  When the size is
  specified, I add FSize blank items ('', nil) to FHash.
  When there is only one item in a bucket, then no chaining
  is necessary and the item is simply assigned to the bucket.
  When a collision occurs, then I create a chain,
  which is another TStringList and add to the chain the item
  currently in the bucket plus the new item.  I can tell that
  something in a bucket is a hash item versus a chain
  if its value is not nil but its key is ''.  (That's why
  I don't allow blank keys.) This scheme saves having to create
  lots of chains with just 1 item in them (which should be
  the case if the hash is efficient).  When the last item
  is deleted from a chain, the chain is freed and the bucket
  becomes available.

  Caveat: one side effect of this scheme is that you cannot hash
  a value that is indistinguishable from a nil pointer, i.e.,
  you canNOT do something like this:

  value:=0;
  myHash['foo']:=Pointer( value);

  There are at least two ways to handle the above: 1) if all your
  values are positive, simply add 1 to value before assigning it
  to the hash and subtract 1 when retrieving it from the hash;
  or 2) put a record or class wrapper around the value.


  I use a const array of 288 prime numbers to determine the size
  of the hash table.  So if you request a size of say, 600, then
  I use the next greatest prime number in the array as the
  actual size, namely, 607.  If the requested size N is larger than 
  the greatest number in the array, then N itself is used as the size.
  This is rather awkward, I know.  If someone has a better idea,
  please let me know.

*)

interface

uses Classes, SysUtils;

const
   NumOfPrimes = 288;
   Primes: array[1..NumOfPrimes] of integer =
    (17     ,31     ,127    ,233    ,353    ,467,
     607    ,739    ,877    ,1019   ,1153   ,1297   ,
     1381   ,1523   ,1663   ,1823   ,1993   ,2131   ,
     2221   ,2371   ,2539   ,2689   ,2833   ,3001   ,
     3083   ,3259   ,3433   ,3581   ,3733   ,3911   ,
     4001   ,4153   ,4327   ,4507   ,4663   ,4861   ,
     4943   ,5099   ,5281   ,5449   ,5641   ,5801   ,
     5861   ,6067   ,6229   ,6373   ,6577   ,6763   ,
     6841   ,7001   ,7211   ,7417   ,7573   ,7727   ,
     7841   ,8039   ,8221   ,8389   ,8599   ,8747   ,
     8837   ,9013   ,9203   ,9391   ,9539   ,9739   ,
     9817   ,10009  ,10181  ,10357  ,10589  ,10753  ,
     10861  ,11069  ,11257  ,11447  ,11677  ,11839  ,
     11939  ,12113  ,12301  ,12491  ,12647  ,12841  ,
     12941  ,13121  ,13313  ,13513  ,13709  ,13883  ,
     13997  ,14207  ,14423  ,14621  ,14771  ,14951  ,
     15077  ,15263  ,15413  ,15619  ,15787  ,15973  ,
     16073  ,16273  ,16487  ,16693  ,16921  ,17099  ,
     17203  ,17393  ,17579  ,17789  ,17977  ,18149  ,
     18251  ,18433  ,18661  ,18911  ,19139  ,19333  ,
     19427  ,19577  ,19801  ,19993  ,20161  ,20359  ,
     20477  ,20707  ,20899  ,21089  ,21283  ,21493  ,
     21569  ,21757  ,21961  ,22129  ,22343  ,22549  ,
     22651  ,22853  ,23039  ,23209  ,23459  ,23633  ,
     23747  ,23911  ,24097  ,24317  ,24527  ,24781  ,
     24889  ,25111  ,25307  ,25523  ,25703  ,25919  ,
     26003  ,26209  ,26407  ,26641  ,26813  ,26993  ,
     27091  ,27337  ,27581  ,27773  ,27953  ,28163  ,
     28289  ,28513  ,28657  ,28843  ,29063  ,29269  ,
     29383  ,29581  ,29833  ,30071  ,30253  ,30491  ,
     30577  ,30809  ,31013  ,31189  ,31379  ,31607  ,
     31723  ,31981  ,32173  ,32359  ,32533  ,32719  ,
     32833  ,33029  ,33223  ,33457  ,33617  ,33809  ,
     33911  ,34159  ,34351  ,34543  ,34747  ,34963  ,
     35089  ,35311  ,35521  ,35771  ,35977  ,36187  ,
     36293  ,36523  ,36697  ,36887  ,37061  ,37313  ,
     37409  ,37579  ,37831  ,38053  ,38287  ,38557  ,
     38651  ,38833  ,39043  ,39229  ,39439  ,39671  ,
     39779  ,39979  ,40177  ,40459  ,40693  ,40879  ,
     40993  ,41189  ,41389  ,41609  ,41809  ,41983  ,
     42083  ,42293  ,42461  ,42683  ,42853  ,43063  ,
     43201  ,43457  ,43669  ,43943  ,44111  ,44281  ,
     44453  ,44647  ,44851  ,45083  ,45317  ,45541  ,
     45659  ,45863  ,46103  ,46327  ,46559  ,46751  ,
     46853  ,47123  ,47339  ,47533  ,47737  ,47947  ,
     48073  ,48311  ,48523  ,48751  ,48947  ,49139  ,
     49223  ,49451  ,49667  ,49871  ,50077  ,50287  ,
     50383  ,50599  ,50873  ,51131  ,51341  ,51503 );

type

  EHashError = class( Exception);

  THashFunc  = function( Sender: TObject; const Key: string): longint;

  THashItem = record
    Key: string;
    Value: Pointer;
  end;

  THash = class
   private
     FHash: TStringList;
     FHashFunc: THashFunc;
     // max # of buckets, occupied buckets,
     // # of chains created, # of hash items added
     FSize,FUsed,FChains,FCount: integer;
     // bucket and chain index of current item
     // (used for .First and .Next method to
     FBIdx, FCIdx: integer;
     FAutoGrow: boolean;

     function h( const Key: string): longint;
     function FindFirst( Start: integer): THashItem;
     function ChainOf( B: integer): TStringList;
     function HasChain( B: integer): boolean;
     procedure Rebuild( sz: integer);
     procedure CheckGrowth;

     procedure SetSize( sz: integer);
     procedure SetAutoGrow( flag: boolean);
     procedure SetHashFunc( Fn: THashFunc);
     function GetAvgChainSize: real;
     function GetLongestChain: integer;
     function GetiCount( B: integer): integer;

   public
     constructor Create;
     destructor Destroy; override;
     function Delete( const Key: string): boolean;
     function Add( const Key: string; Val: Pointer): boolean;
     function Get( const Key: string): Pointer;
     procedure Put( const Key: string; Val: Pointer);
     procedure Clear;
     function First: THashItem;
     function Next: THashItem;

     property Size:integer read FSize write SetSize;
     property Items[const Key: string]: Pointer read Get write Put; default;
     property iCount[ B: integer]: integer read GetiCount;
     property Count:integer read FCount;
     property HashFunc: THashFunc read FHashFunc write SetHashFunc;
     property Chains:integer read FChains;
     property Used:integer read FUsed;
     property LongestChain: integer read GetLongestChain;
     property AvgChainSize: real read GetAvgChainSize;
     property AutoGrow: boolean read FAutoGrow write SetAutoGrow;

  end;

implementation

// helper functions; should really be macros, but...
function THash.ChainOf( B: integer): TStringList;
begin
  Result:=TStringList( FHash.Objects[B]);
end;

function THash.HasChain( B: integer): boolean;
begin
  Result:=(FHash.Objects[B]<>nil) and (FHash[B]='');
end;

constructor THash.Create;
begin
  inherited Create;
  FHash:=TStringList.Create;
  FSize:=0;
  FCount:=0;
  FUsed:=0;
  FChains:=0;
  FHashFunc:=nil;
  FBIdx:=-1;
  FCIdx:=-1;
  FAutoGrow:=true;
  SetSize( Primes[1]);
end;

procedure THash.Clear;
var i:integer;
begin
  for i:=FHash.Count-1 downto 0 do
    if HasChain( i) then
      ChainOf( i).Free;
  FHash.Clear;
  FSize:=0;
  FCount:=0;
  FUsed:=0;
  FChains:=0;
end;

destructor THash.Destroy;
begin
  Clear;
  FHash.Free;
  inherited Destroy;
end;

procedure THash.SetSize( sz: integer);
var i:integer;
begin
  if (sz<=0) or (sz>MaxListSize) or (sz=FSize) then
    exit;
  i:=1;
  while (i<NumOfPrimes) and (sz>Primes[i]) do Inc( i);
  if sz<Primes[i] then sz:=Primes[i];
  if (FCount<>0) then
    Rebuild( sz)
  else
    begin
      FHash.Capacity:=sz;
      for i:=1 to sz do
        FHash.AddObject( '', nil);
      FSize:=sz;
    end;
end;

procedure THash.Rebuild( sz: integer);
var OldItems: TStringList; HI: THashItem; i:integer;
begin
  OldItems:=TStringList.Create;
  Try
    OldItems.Capacity:=FCount;
    HI:=First;
    while HI.Value<>nil do
    begin
      OldItems.AddObject( HI.Key, HI.Value);
      HI:=Next;
    end;
    Clear;
    SetSize( sz);
    with OldItems do
      for i:=0 to Count-1 do
        Self.Add( Strings[i], Objects[i]);
  Finally
    OldItems.Free;
  end;
end;


procedure THash.SetAutoGrow( flag: boolean);
begin
  if flag<>FAutoGrow then
  begin
    FAutoGrow:=flag;
    if flag then CheckGrowth;
  end;
end;

procedure THash.SetHashFunc( Fn: THashFunc);
begin
  FHashFunc:=Fn;
  if FCount>0 then Rebuild( FSize);
end;

procedure THash.CheckGrowth;
var i,n:integer;
begin
  if FAutoGrow and (FCount>2*FSize) then
    SetSize( 2*FSize);
end;

function THash.h( const Key: string): longint;
var i,c,b,S:integer;
begin
  if Assigned( FHashFunc) then
    Result:=FHashFunc( Self, Key)
  else
  begin
    result:=Length( Key);
    for i:=1 to Length( Key) do
      Result:= ((Result shl 5) xor (Result shr 27)) xor Ord(Key[i]);
  end;
  if Result<0 then Result:=-Result;
  Result:=Result mod FSize;
end;

function THash.Get( const Key: string): Pointer;
var B,i: integer;
begin
  Result := nil;
  B := h( Key);
  if (FHash.Objects[B]=nil) then
     exit;
  if FHash[B]='' then
    with ChainOf( B) do
    begin
      i:=IndexOf( Key);
      if i>=0 then Result:=Objects[i];
    end
  else
    if FHash[B]=Key then
      Result:=FHash.Objects[B];
end;

// add is like put, but does not check for existing item
function THash.Add( const Key: string; Val: Pointer): boolean;
var B: integer; Chn: TStringList;
begin
  Result:=false;
  if (Key='') or (Val=nil) then
    exit;
  B:=h( Key);
  if FHash.Objects[B]=nil then
  begin
    // open bucket! just put it in
    FHash[B]:=Key;
    FHash.Objects[B]:=Val;
    Inc( FCount);
    Inc( FUsed);
    Result:=true;
  end else
  begin
    // occupied bucket, more complicated
    // does chain already exist?
    if FHash[B]='' then
    with ChainOf( B) do
    begin
      AddObject( Key, Val);
      Inc( FCount);
      Result:=true;
    end else
    // no chain yet
    begin
      // create chain
      Chn:=TStringList.Create;
      with Chn do
      begin
        // add the item in the bucket slot to the chain
        AddObject( FHash[B], FHash.Objects[B]);
        // now add the new item
        AddObject( Key, Val);
        Inc( FCount);
        Inc( FChains);
        Result:=true;
      end;
      // bucket slot now points to chain
      FHash[B]:='';
      FHash.Objects[B]:=Chn;
    end;
  end;
  if Result then CheckGrowth;
end;

procedure THash.Put( const Key: string; Val: Pointer);
var B,i: integer; Chn: TStringList;
begin
  if Key='' then exit;
  B:=h( Key);
  if FHash.Objects[B]=nil then
  begin
    // open bucket! just put it in
    if Val<>nil then
    begin
      FHash[B]:=Key;
      FHash.Objects[B]:=Val;
      Inc( FCount);
      Inc( FUsed);
      CheckGrowth;
    end;
  end else
  begin
    // occupied bucket, more complicated
    if Val<>nil then
    begin
      // is item in the bucket slot itself? change it
      if FHash[B]=Key then
         FHash.Objects[B]:=Val
      else
      // no, it's not the one in the bucket slot
      // does chain already exist?
      if FHash[B]='' then
      with ChainOf( B) do
      begin
        // is item in the chain?
        i:=IndexOf( Key);
        // just change the Val
        if i>=0 then
          Objects[i]:=Val
        else
          // no, add it to chain
          begin
            AddObject( Key, Val);
            Inc( FCount);
            CheckGrowth;
          end;
      end else
      // no chain yet
      begin
        // create chain
        Chn:=TStringList.Create;
        with Chn do
        begin
          // add the item in the bucket slot to the chain
          AddObject( FHash[B], FHash.Objects[B]);
          // now add the new item
          AddObject( Key, Val);
          Inc( FCount);
          Inc( FChains);
        end;
        // bucket slot now points to chain
        FHash[B]:='';
        FHash.Objects[B]:=Chn;
        CheckGrowth;
      end;
    end else
    // value is nil, must want to delete it
      Delete( Key);
  end;
end;

function THash.Delete( const Key: string): boolean;
var B,i: integer;
begin
  Result:=false;
  B:= h( Key);
  if (FHash.Objects[B]=nil) then
    exit;
  // is it in the bucket?
  if FHash[B]=Key then
  begin
    FHash[B]:='';
    FHash.Objects[B]:=nil;
    Dec( FCount);
    Dec( FUsed);
    Result:=true;
  end else
  if FHash[B]='' then
  with ChainOf( B) do
  begin
    // is it in the chain?
    i:=IndexOf( Key);
    if i>=0 then
    begin
      Delete( i);
      Dec( FCount);
      Result:=true;
      // did we delete the last item in the chain?
      if Count=0 then
      begin
        // yes, delete the chain, too
        Free;
        FHash[B]:='';
        FHash.Objects[B]:=nil;
        Dec( FUsed);
        Dec( FChains);
      end;
    end;
  end;
end;


function THash.FindFirst( Start: integer): THashItem;
var i:integer;
begin
  Result.Key:='';
  Result.Value:=nil;
  for i:=Start to FHash.Count-1 do
    if (FHash.Objects[i]<>nil) then
     if (FHash[i]='') then
     with ChainOf( i) do
      begin
        Result.Key:=Strings[0];
        Result.Value:=Objects[0];
        FBIdx:=i;
        FCIdx:=1;
        break;
      end else
     begin
       Result.Key:=FHash[i];
       Result.Value:=FHash.Objects[i];
       FBIdx:=i+1;
       FCIdx:=-1;
       break;
     end;
end;

function THash.First: THashItem;
begin
  Result:=FindFirst( 0);
end;

function THash.Next: THashItem;
begin
  Result.Key:='';
  Result.Value:=nil;
  if FBIdx<0 then
    Result:=First
  else if FBIdx>=FSize-1 then
    exit
  else
    begin
      if (FHash.Objects[FBIdx]<>nil) then
      begin
        if (FHash[FBIdx]<>'') then
        begin
          Result.Key:=FHash[FBIdx];
          Result.Value:=FHash.Objects[FBIdx];
          Inc( FBIdx);
          FCIdx:=-1;
        end else
        if FCIdx<ChainOf( FBIdx).Count then
        begin
          if FCIdx<0 then FCIdx:=0;
          with ChainOf( FBIdx) do
          begin
            Result.Key:=Strings[FCIdx];
            Result.Value:=Objects[FCIdx];
            Inc( FCIdx);
          end;
        end;
      end;
      if Result.Value=nil then
        Result:=FindFirst( FBIdx+1);
    end;
end;

function THash.GetLongestChain: integer;
var i,c:integer;
begin
  Result:=0;
  for i:=0 to FHash.Count-1 do
    if HasChain( i) then
    begin
      c:=ChainOf( i).Count;
      if Result<c then Result:=c;
    end;
end;

function THash.GetAvgChainSize: real;
var i,n,c:integer;
begin
  c:=0;
  n:=0;
  for i:=0 to FHash.Count-1 do
    if HasChain( i) then
    begin
      Inc( c);
      Inc( n, ChainOf( i).Count);
    end;
  Result:=n/c;
end;

function THash.GetiCount( B:integer) : integer;
begin
  Result:=-1;
  if (B>=0) and (B<FSize) then
  begin
    if FHash.Objects[B]=nil then
      Result:=0
    else if FHash[B]<>'' then
      Result:=1
    else Result:=ChainOf( B).Count;
  end;
end;
 
end.
